/****************************************************************************
**
** Copyright (C) 2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com, author Milian Wolff <milian.wolff@kdab.com>
** Contact: http://www.qt.io/licensing/
**
** This file is part of the QtWebChannel module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL21$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see http://www.qt.io/terms-conditions. For further
** information use the contact form at http://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 or version 3 as published by the Free
** Software Foundation and appearing in the file LICENSE.LGPLv21 and
** LICENSE.LGPLv3 included in the packaging of this file. Please review the
** following information to ensure the GNU Lesser General Public License
** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** As a special exception, The Qt Company gives you certain additional
** rights. These rights are described in The Qt Company LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include "qwebchannelwebkittransport_p.h"

#include <QJsonObject>
#include <QJsonDocument>

#include "qquickwebview_p.h"

QWebChannelWebKitTransport::QWebChannelWebKitTransport(QQuickWebViewExperimental* experimental)
    : QWebChannelAbstractTransport(experimental)
    , m_experimental(experimental)
{
}

void QWebChannelWebKitTransport::sendMessage(const QJsonObject& message)
{
    const QByteArray data = QJsonDocument(message).toJson(QJsonDocument::Compact);
    m_experimental->postQtWebChannelTransportMessage(data);
}

void QWebChannelWebKitTransport::receiveMessage(const QByteArray& message)
{
    QJsonParseError error;
    const QJsonDocument doc = QJsonDocument::fromJson(message, &error);
    if (error.error != QJsonParseError::NoError) {
        qWarning() << "Failed to parse the client WebKit QWebChannel message as JSON: " << message
                   << "Error message is:" << error.errorString();
        return;
    } else if (!doc.isObject()) {
        qWarning() << "Received WebKit QWebChannel message is not a JSON object: " << message;
        return;
    }
    emit messageReceived(doc.object(), this);
}
